@@ -412,40 +412,6 @@ MapMat<eT>::speye(const SizeMat& s)
 
 
 
-template<typename eT>
-arma_inline
-MapMat_elem<eT>
-MapMat<eT>::elem(const uword index, state_type& sync_state, uword& n_nonzero)
-  {
-  return MapMat_elem<eT>(*this, index, sync_state, n_nonzero);
-  }
-
-
-
-template<typename eT>
-arma_inline
-MapMat_elem<eT>
-MapMat<eT>::elem(const uword in_row, const uword in_col, state_type& sync_state, uword& n_nonzero)
-  {
-  const uword index = (n_rows * in_col) + in_row;
-  
-  return MapMat_elem<eT>(*this, index, sync_state, n_nonzero);
-  }
-
-
-
-template<typename eT>
-arma_inline
-MapMat_svel<eT>
-MapMat<eT>::svel(const uword in_row, const uword in_col, state_type& sync_state, uword& n_nonzero, uword& sv_n_nonzero)
-  {
-  const uword index = (n_rows * in_col) + in_row;
-  
-  return MapMat_svel<eT>(*this, index, sync_state, n_nonzero, sv_n_nonzero);
-  }
-
-
-
 template<typename eT>
 arma_inline
 arma_warn_unused
@@ -1114,17 +1080,17 @@ MapMat_val<eT>::operator--(int)
 
 
 
-// MapMat_elem
+// SpMat_MapMat_val
 
 
 
 template<typename eT>
 arma_inline
-MapMat_elem<eT>::MapMat_elem(MapMat<eT>& in_parent, const uword in_index, state_type& in_sync_state, uword& in_n_nonzero)
-  : parent    (in_parent    )
-  , index     (in_index     )
-  , sync_state(in_sync_state)
-  , n_nonzero (in_n_nonzero )
+SpMat_MapMat_val<eT>::SpMat_MapMat_val(SpMat<eT>& in_s_parent, MapMat<eT>& in_m_parent, const uword in_row, const uword in_col)
+  : s_parent(in_s_parent)
+  , m_parent(in_m_parent)
+  , row     (in_row     )
+  , col     (in_col     )
   {
   arma_extra_debug_sigprint();
   }
@@ -1133,30 +1099,41 @@ MapMat_elem<eT>::MapMat_elem(MapMat<eT>& in_parent, const uword in_index, state_
 
 template<typename eT>
 arma_inline
-MapMat_elem<eT>::operator eT() const
+SpMat_MapMat_val<eT>::operator eT() const
   {
   arma_extra_debug_sigprint();
   
-  const MapMat<eT>& const_parent = parent;
+  const SpMat<eT>& const_s_parent = s_parent;
   
-  return const_parent.operator[](index);
+  return const_s_parent.get_value(row,col);  // use the const version of get_value()
+  
+  // s_parent.sync_cache();
+  // 
+  // const MapMat<eT>& const_m_parent = m_parent;
+  // 
+  // const uword index = (const_m_parent.n_rows * col) + row;
+  //   
+  // return const_m_parent.operator[](index);
   }
 
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator=(const MapMat_elem<eT>& x)
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator=(const SpMat_MapMat_val<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  const eT in_val = eT(x);
+  s_parent.sync_cache();
   
-  parent.set_val(index, in_val);
+  const eT    in_val = eT(x);
+  const uword index  = (m_parent.n_rows * col) + row;
   
-  sync_state = 1;
-  n_nonzero  = parent.get_n_nonzero();
+  m_parent.set_val(index, in_val);
+  
+  s_parent.sync_state = 1;
+  access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
   
   return *this;
   }
@@ -1164,30 +1141,38 @@ MapMat_elem<eT>::operator=(const MapMat_elem<eT>& x)
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator=(const eT in_val)
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  parent.set_val(index, in_val);
+  s_parent.sync_cache();
   
-  sync_state = 1;
-  n_nonzero  = parent.get_n_nonzero();
+  const uword index = (m_parent.n_rows * col) + row;
   
+  m_parent.set_val(index, in_val);
+  
+  s_parent.sync_state = 1;
+  access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
+ 
   return *this;
   }
 
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator+=(const eT in_val)
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator+=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   if(in_val != eT(0))
     {
@@ -1197,8 +1182,8 @@ MapMat_elem<eT>::operator+=(const eT in_val)
     
     if(val == eT(0))  { map_ref.erase(index); }
     
-    sync_state = 1;
-    n_nonzero  = parent.get_n_nonzero();
+    s_parent.sync_state = 1;
+    access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
     }
   
   return *this;
@@ -1207,13 +1192,17 @@ MapMat_elem<eT>::operator+=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator-=(const eT in_val)
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator-=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   if(in_val != eT(0))
     {
@@ -1223,8 +1212,8 @@ MapMat_elem<eT>::operator-=(const eT in_val)
     
     if(val == eT(0))  { map_ref.erase(index); }
     
-    sync_state = 1;
-    n_nonzero  = parent.get_n_nonzero();
+    s_parent.sync_state = 1;
+    access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
     }
   
   return *this;
@@ -1233,13 +1222,17 @@ MapMat_elem<eT>::operator-=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator*=(const eT in_val)
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator*=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   typename MapMat<eT>::map_type::iterator it     = map_ref.find(index);
   typename MapMat<eT>::map_type::iterator it_end = map_ref.end();
@@ -1259,8 +1252,8 @@ MapMat_elem<eT>::operator*=(const eT in_val)
       map_ref.erase(it);
       }
     
-    sync_state = 1;
-    n_nonzero  = parent.get_n_nonzero();
+    s_parent.sync_state = 1;
+    access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
     }
   
   return *this;
@@ -1269,13 +1262,17 @@ MapMat_elem<eT>::operator*=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator/=(const eT in_val)
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator/=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   typename MapMat<eT>::map_type::iterator it     = map_ref.find(index);
   typename MapMat<eT>::map_type::iterator it_end = map_ref.end();
@@ -1288,8 +1285,8 @@ MapMat_elem<eT>::operator/=(const eT in_val)
     
     if(val == eT(0))  { map_ref.erase(it); }
     
-    sync_state = 1;
-    n_nonzero  = parent.get_n_nonzero();
+    s_parent.sync_state = 1;
+    access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
     }
   else
     {
@@ -1299,10 +1296,10 @@ MapMat_elem<eT>::operator/=(const eT in_val)
     
     if(val != eT(0))
       {
-      parent.set_val(index, val);
+      m_parent.set_val(index, val);
       
-      sync_state = 1;
-      n_nonzero  = parent.get_n_nonzero();
+      s_parent.sync_state = 1;
+      access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
       }
     }
   
@@ -1312,13 +1309,17 @@ MapMat_elem<eT>::operator/=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator++()
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator++()
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1326,8 +1327,8 @@ MapMat_elem<eT>::operator++()
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
-  n_nonzero  = parent.get_n_nonzero();
+  s_parent.sync_state = 1;
+  access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
   
   return *this;
   }
@@ -1335,13 +1336,17 @@ MapMat_elem<eT>::operator++()
 
 
 template<typename eT>
-arma_inline
+inline
 eT
-MapMat_elem<eT>::operator++(int)
+SpMat_MapMat_val<eT>::operator++(int)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1351,8 +1356,8 @@ MapMat_elem<eT>::operator++(int)
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
-  n_nonzero  = parent.get_n_nonzero();
+  s_parent.sync_state = 1;
+  access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
   
   return old_val;
   }
@@ -1360,13 +1365,17 @@ MapMat_elem<eT>::operator++(int)
 
 
 template<typename eT>
-arma_inline
-MapMat_elem<eT>&
-MapMat_elem<eT>::operator--()
+inline
+SpMat_MapMat_val<eT>&
+SpMat_MapMat_val<eT>::operator--()
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1374,8 +1383,8 @@ MapMat_elem<eT>::operator--()
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
-  n_nonzero  = parent.get_n_nonzero();
+  s_parent.sync_state = 1;
+  access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
   
   return *this;
   }
@@ -1383,13 +1392,17 @@ MapMat_elem<eT>::operator--()
 
 
 template<typename eT>
-arma_inline
+inline
 eT
-MapMat_elem<eT>::operator--(int)
+SpMat_MapMat_val<eT>::operator--(int)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  s_parent.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1399,8 +1412,8 @@ MapMat_elem<eT>::operator--(int)
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
-  n_nonzero  = parent.get_n_nonzero();
+  s_parent.sync_state = 1;
+  access::rw(s_parent.n_nonzero) = m_parent.get_n_nonzero();
   
   return old_val;
   }
@@ -1409,18 +1422,17 @@ MapMat_elem<eT>::operator--(int)
 
 
 
-// MapMat_svel
+// SpSubview_MapMat_val
 
 
 
 template<typename eT>
 arma_inline
-MapMat_svel<eT>::MapMat_svel(MapMat<eT>& in_parent, const uword in_index, state_type& in_sync_state, uword& in_n_nonzero, uword& in_sv_n_nonzero)
-  : parent      (in_parent      )
-  , index       (in_index       )
-  , sync_state  (in_sync_state  )
-  , n_nonzero   (in_n_nonzero   )
-  , sv_n_nonzero(in_sv_n_nonzero)
+SpSubview_MapMat_val<eT>::SpSubview_MapMat_val(SpSubview<eT>& in_v_parent, MapMat<eT>& in_m_parent, const uword in_row, const uword in_col)
+  : v_parent(in_v_parent)
+  , m_parent(in_m_parent)
+  , row     (in_row     )
+  , col     (in_col     )
   {
   arma_extra_debug_sigprint();
   }
@@ -1430,45 +1442,56 @@ MapMat_svel<eT>::MapMat_svel(MapMat<eT>& in_parent, const uword in_index, state_
 template<typename eT>
 arma_inline
 void
-MapMat_svel<eT>::update_n_nonzeros()
+SpSubview_MapMat_val<eT>::update_n_nonzeros()
   {
   arma_extra_debug_sigprint();
   
-  const uword old_n_nonzero = n_nonzero;
+  const uword old_n_nonzero = v_parent.m.n_nonzero;
   
-  n_nonzero = parent.get_n_nonzero();
+  access::rw(v_parent.m.n_nonzero) = m_parent.get_n_nonzero();
   
-       if(n_nonzero > old_n_nonzero)  { ++sv_n_nonzero; }
-  else if(n_nonzero < old_n_nonzero)  { --sv_n_nonzero; }
+       if(v_parent.m.n_nonzero > old_n_nonzero)  { access::rw(v_parent.n_nonzero)++; }
+  else if(v_parent.m.n_nonzero < old_n_nonzero)  { access::rw(v_parent.n_nonzero)--; }
   }
 
 
 
 template<typename eT>
 arma_inline
-MapMat_svel<eT>::operator eT() const
+SpSubview_MapMat_val<eT>::operator eT() const
   {
   arma_extra_debug_sigprint();
   
-  const MapMat<eT>& const_parent = parent;
+  const SpMat<eT>& const_s_parent = v_parent.m;
   
-  return const_parent.operator[](index);
+  return const_s_parent.get_value(row,col);  // use the const version of get_value()
+  
+  // v_parent.m.sync_cache();
+  // 
+  // const MapMat<eT>& const_m_parent = m_parent;
+  // 
+  // const uword index = (const_m_parent.n_rows * col) + row;
+  //   
+  // return const_m_parent.operator[](index);
   }
 
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator=(const MapMat_svel<eT>& x)
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator=(const SpSubview_MapMat_val<eT>& x)
   {
   arma_extra_debug_sigprint();
   
-  const eT in_val = eT(x);
+  v_parent.m.sync_cache();
   
-  parent.set_val(index, in_val);
+  const eT    in_val = eT(x);
+  const uword index  = (m_parent.n_rows * col) + row;
   
-  sync_state = 1;
+  m_parent.set_val(index, in_val);
+  
+  v_parent.m.sync_state = 1;
   update_n_nonzeros();
   
   return *this;
@@ -1477,15 +1500,19 @@ MapMat_svel<eT>::operator=(const MapMat_svel<eT>& x)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator=(const eT in_val)
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  parent.set_val(index, in_val);
+  v_parent.m.sync_cache();
   
-  sync_state = 1;
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  m_parent.set_val(index, in_val);
+  
+  v_parent.m.sync_state = 1;
   update_n_nonzeros();
   
   return *this;
@@ -1494,13 +1521,17 @@ MapMat_svel<eT>::operator=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator+=(const eT in_val)
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator+=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   if(in_val != eT(0))
     {
@@ -1510,7 +1541,7 @@ MapMat_svel<eT>::operator+=(const eT in_val)
     
     if(val == eT(0))  { map_ref.erase(index); }
     
-    sync_state = 1;
+    v_parent.m.sync_state = 1;
     update_n_nonzeros();
     }
   
@@ -1520,13 +1551,17 @@ MapMat_svel<eT>::operator+=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator-=(const eT in_val)
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator-=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   if(in_val != eT(0))
     {
@@ -1536,7 +1571,7 @@ MapMat_svel<eT>::operator-=(const eT in_val)
     
     if(val == eT(0))  { map_ref.erase(index); }
     
-    sync_state = 1;
+    v_parent.m.sync_state = 1;
     update_n_nonzeros();
     }
   
@@ -1546,13 +1581,17 @@ MapMat_svel<eT>::operator-=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator*=(const eT in_val)
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator*=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   typename MapMat<eT>::map_type::iterator it     = map_ref.find(index);
   typename MapMat<eT>::map_type::iterator it_end = map_ref.end();
@@ -1572,7 +1611,7 @@ MapMat_svel<eT>::operator*=(const eT in_val)
       map_ref.erase(it);
       }
     
-    sync_state = 1;
+    v_parent.m.sync_state = 1;
     update_n_nonzeros();
     }
   
@@ -1582,13 +1621,17 @@ MapMat_svel<eT>::operator*=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator/=(const eT in_val)
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator/=(const eT in_val)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   typename MapMat<eT>::map_type::iterator it     = map_ref.find(index);
   typename MapMat<eT>::map_type::iterator it_end = map_ref.end();
@@ -1601,7 +1644,7 @@ MapMat_svel<eT>::operator/=(const eT in_val)
     
     if(val == eT(0))  { map_ref.erase(it); }
     
-    sync_state = 1;
+    v_parent.m.sync_state = 1;
     update_n_nonzeros();
     }
   else
@@ -1612,9 +1655,9 @@ MapMat_svel<eT>::operator/=(const eT in_val)
     
     if(val != eT(0))
       {
-      parent.set_val(index, val);
+      m_parent.set_val(index, val);
       
-      sync_state = 1;
+      v_parent.m.sync_state = 1;
       update_n_nonzeros();
       }
     }
@@ -1625,13 +1668,17 @@ MapMat_svel<eT>::operator/=(const eT in_val)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator++()
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator++()
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1639,7 +1686,7 @@ MapMat_svel<eT>::operator++()
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
+  v_parent.m.sync_state = 1;
   update_n_nonzeros();
   
   return *this;
@@ -1648,13 +1695,17 @@ MapMat_svel<eT>::operator++()
 
 
 template<typename eT>
-arma_inline
+inline
 eT
-MapMat_svel<eT>::operator++(int)
+SpSubview_MapMat_val<eT>::operator++(int)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1664,7 +1715,7 @@ MapMat_svel<eT>::operator++(int)
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
+  v_parent.m.sync_state = 1;
   update_n_nonzeros();
   
   return old_val;
@@ -1673,13 +1724,17 @@ MapMat_svel<eT>::operator++(int)
 
 
 template<typename eT>
-arma_inline
-MapMat_svel<eT>&
-MapMat_svel<eT>::operator--()
+inline
+SpSubview_MapMat_val<eT>&
+SpSubview_MapMat_val<eT>::operator--()
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1687,7 +1742,7 @@ MapMat_svel<eT>::operator--()
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
+  v_parent.m.sync_state = 1;
   update_n_nonzeros();
   
   return *this;
@@ -1696,13 +1751,17 @@ MapMat_svel<eT>::operator--()
 
 
 template<typename eT>
-arma_inline
+inline
 eT
-MapMat_svel<eT>::operator--(int)
+SpSubview_MapMat_val<eT>::operator--(int)
   {
   arma_extra_debug_sigprint();
   
-  typename MapMat<eT>::map_type& map_ref = *(parent.map_ptr);
+  v_parent.m.sync_cache();
+  
+  const uword index = (m_parent.n_rows * col) + row;
+  
+  typename MapMat<eT>::map_type& map_ref = *(m_parent.map_ptr);
   
   eT& val = map_ref.operator[](index);  // creates the element if it doesn't exist
   
@@ -1712,7 +1771,7 @@ MapMat_svel<eT>::operator--(int)
   
   if(val == eT(0))  { map_ref.erase(index); }
   
-  sync_state = 1;
+  v_parent.m.sync_state = 1;
   update_n_nonzeros();
   
   return old_val;
