@@ -30,7 +30,7 @@ SpMat<eT>::iterator_base::iterator_base()
   , internal_col(0)
   , internal_pos(0)
   {
-  // Technically this iterator is invalid (it may not point to a real element)
+  // Technically this iterator is invalid (it does not point to a valid element)
   }
 
 
@@ -42,7 +42,7 @@ SpMat<eT>::iterator_base::iterator_base(const SpMat<eT>& in_M)
   , internal_col(0)
   , internal_pos(0)
   {
-  // Technically this iterator is invalid (it may not point to a real element)
+  // Technically this iterator is invalid (it may not point to a valid element)
   }
 
 
@@ -60,8 +60,7 @@ SpMat<eT>::iterator_base::iterator_base(const SpMat<eT>& in_M, const uword in_co
 
 
 template<typename eT>
-inline
-arma_hot
+arma_inline
 eT
 SpMat<eT>::iterator_base::operator*() const
   {
@@ -172,7 +171,7 @@ SpMat<eT>::const_iterator::operator++()
 
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::const_iterator
 SpMat<eT>::const_iterator::operator++(int)
   {
@@ -191,19 +190,11 @@ arma_hot
 typename SpMat<eT>::const_iterator&
 SpMat<eT>::const_iterator::operator--()
   {
-  //iterator_base::M.print("M");
-  
-  // printf("decrement from %d, %d, %d\n", iterator_base::internal_pos, iterator_base::internal_col, iterator_base::row());
-  
   --iterator_base::internal_pos;
   
-  // printf("now pos %d\n", iterator_base::internal_pos);
-
   // First, see if we moved back a column.
   while (iterator_base::internal_pos < iterator_base::M->col_ptrs[iterator_base::internal_col])
     {
-    // printf("colptr %d (col %d)\n", iterator_base::M.col_ptrs[iterator_base::internal_col], iterator_base::internal_col);
-    
     --iterator_base::internal_col;
     }
 
@@ -215,7 +206,7 @@ SpMat<eT>::const_iterator::operator--()
 
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::const_iterator
 SpMat<eT>::const_iterator::operator--(int)
   {
@@ -349,7 +340,7 @@ SpMat<eT>::iterator::operator++()
 
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::iterator
 SpMat<eT>::iterator::operator++(int)
   {
@@ -376,7 +367,7 @@ SpMat<eT>::iterator::operator--()
 
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::iterator
 SpMat<eT>::iterator::operator--(int)
   {
@@ -415,11 +406,14 @@ SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword i
   , internal_row(0)
   , actual_pos(0)
   {
-  // Corner case for empty matrix.
-  if(in_M.n_nonzero == 0)
+  // Corner case for the end of a matrix.
+  if (initial_pos == in_M.n_nonzero)
     {
     iterator_base::internal_col = 0;
     internal_row = in_M.n_rows;
+    actual_pos = in_M.n_nonzero;
+    iterator_base::internal_pos = in_M.n_nonzero;
+
     return;
     }
 
@@ -432,46 +426,58 @@ SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword i
   // row 0 (and add to our sum), then in row 1, and so forth, until we get to
   // the desired position.
   uword cur_pos = std::numeric_limits<uword>::max(); // Invalid value.
-  uword cur_row = 0;
-  uword cur_col = 0;
+  uword cur_actual_pos = 0;
 
-  while(true) // This loop is terminated from the inside.
+  for (uword row = 0; row < iterator_base::M->n_rows; ++row)
     {
-    // Is there anything in the column we are looking at?
-    for (uword ind = 0; ((iterator_base::M->col_ptrs[cur_col] + ind < iterator_base::M->col_ptrs[cur_col + 1]) && (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] <= cur_row)); ind++)
+    for (uword col = 0; col < iterator_base::M->n_cols; ++col)
       {
-      // There is something in this column.  Is it in the row we are looking at?
-      const uword row_index = iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind];
-      if (row_index == cur_row)
+      // Find the first element with row greater than or equal to in_row.
+      const uword      col_offset = iterator_base::M->col_ptrs[col    ];
+      const uword next_col_offset = iterator_base::M->col_ptrs[col + 1];
+
+      const uword* start_ptr = &iterator_base::M->row_indices[     col_offset];
+      const uword*   end_ptr = &iterator_base::M->row_indices[next_col_offset];
+
+      if (start_ptr != end_ptr)
         {
-        // Yes, it is what we are looking for.  Increment our current position.
-        if (++cur_pos == iterator_base::internal_pos)   // TODO: HACK: if cur_pos is std::numeric_limits<uword>::max(), ++cur_pos relies on a wraparound/overflow, which is not portable
+        const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, row);
+
+        // This is the number of elements in the column with row index less than
+        // in_row.
+        const uword offset = (pos_ptr - start_ptr);
+
+        if (iterator_base::M->row_indices[col_offset + offset] == row)
           {
-          actual_pos = iterator_base::M->col_ptrs[cur_col] + ind;
-          internal_row = cur_row;
-          iterator_base::internal_col = cur_col;
+          cur_actual_pos = col_offset + offset;
 
-          return;
+          // Increment position portably.
+          if (cur_pos == std::numeric_limits<uword>::max())
+            cur_pos = 0;
+          else
+            ++cur_pos;
+
+          // Do we terminate?
+          if (cur_pos == initial_pos)
+            {
+            internal_row = row;
+            iterator_base::internal_col = col;
+            iterator_base::internal_pos = cur_pos;
+            actual_pos = cur_actual_pos;
+
+            return;
+            }
           }
-
-        // We are done with this column.  Break to the column incrementing code (directly below).
-        break;
         }
-      else if(row_index > cur_row)
-        {
-        break; // Can't be in this column.
-        }
-      }
-
-    cur_col++; // Done with the column.  Move on.
-    if (cur_col == iterator_base::M->n_cols)
-      {
-      // We are out of columns.  Loop back to the beginning and look on the
-      // next row.
-      cur_col = 0;
-      cur_row++;
       }
     }
+
+  // If we got to here, then we have gone past the end of the matrix.  This
+  // shouldn't happen...
+  iterator_base::internal_pos = iterator_base::M->n_nonzero;
+  iterator_base::internal_col = 0;
+  internal_row = iterator_base::M->n_rows;
+  actual_pos = iterator_base::M->n_nonzero;
   }
 
 
@@ -483,19 +489,64 @@ SpMat<eT>::const_row_iterator::const_row_iterator(const SpMat<eT>& in_M, uword i
   , internal_row(0)
   , actual_pos(0)
   {
-  // This is slow.  It needs to be rewritten.
-  // So we have a destination we want to be just after, but don't know what position that is.  Make another iterator to find out...
-  const_row_iterator it(in_M, 0);
-  while((it.row() < in_row) || ((it.row() == in_row) && (it.col() < in_col)))
+  // Start our search in the given row.  We need to find two things:
+  //
+  //   1. The first nonzero element (iterating by rows) after (in_row, in_col).
+  //   2. The number of nonzero elements (iterating by rows) that come before
+  //      (in_row, in_col).
+  //
+  // We'll find these simultaneously, though we will have to loop over all
+  // columns.
+
+  // This will hold the total number of points with rows less than in_row.
+  uword cur_pos = 0;
+  uword cur_min_row = iterator_base::M->n_rows;
+  uword cur_min_col = 0;
+  uword cur_actual_pos = 0;
+
+  for (uword col = 0; col < iterator_base::M->n_cols; ++col)
     {
-    it++;
+    // Find the first element with row greater than or equal to in_row.
+    const uword      col_offset = iterator_base::M->col_ptrs[col    ];
+    const uword next_col_offset = iterator_base::M->col_ptrs[col + 1];
+
+    const uword* start_ptr = &iterator_base::M->row_indices[     col_offset];
+    const uword*   end_ptr = &iterator_base::M->row_indices[next_col_offset];
+
+    if (start_ptr != end_ptr)
+      {
+      const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, in_row);
+
+      // This is the number of elements in the column with row index less than
+      // in_row.
+      const uword offset = (pos_ptr - start_ptr);
+
+      cur_pos += offset;
+
+      if (pos_ptr != end_ptr)
+        {
+        // This is the row index of the first element in the column with row index
+        // greater than or equal to in_row.
+        if ((*pos_ptr) < cur_min_row)
+          {
+          // If we are in the desired row but before the desired column, we
+          // can't take this.
+          if (col >= in_col)
+            {
+            cur_min_row = (*pos_ptr);
+            cur_min_col = col;
+            cur_actual_pos = col_offset + offset;
+            }
+          }
+        }
+      }
     }
 
-  // Now that it is at the right place, take its position.
-  iterator_base::internal_col = it.internal_col;
-  iterator_base::internal_pos = it.internal_pos;
-  internal_row = it.internal_row;
-  actual_pos = it.actual_pos;
+  // Now we know what the minimum row is.
+  internal_row = cur_min_row;
+  iterator_base::internal_col = cur_min_col;
+  iterator_base::internal_pos = cur_pos;
+  actual_pos = cur_actual_pos;
   }
 
 
@@ -531,38 +582,112 @@ SpMat<eT>::const_row_iterator::operator++()
     {
     internal_row = iterator_base::M->n_rows;
     iterator_base::internal_col = 0;
-    actual_pos = iterator_base::M->n_nonzero;
 
     return *this;
     }
 
-  // Otherwise, we need to search.
-  uword cur_col = iterator_base::internal_col;
-  uword cur_row = internal_row;
+  // Otherwise, we need to search.  We can start in the next column and use
+  // lower_bound() to find the next element.
+  uword next_min_row = iterator_base::M->n_rows;
+  uword next_min_col = iterator_base::M->n_cols;
+  uword next_actual_pos = 0;
 
-  while (true) // This loop is terminated from the inside.
+  // Search from the current column to the end of the matrix.
+  for (uword col = iterator_base::internal_col + 1; col < iterator_base::M->n_cols; ++col)
     {
-    // Increment the current column and see if we are now on a new row.
-    if (++cur_col == iterator_base::M->n_cols)
-      {
-      cur_col = 0;
-      cur_row++;
-      }
+    // Find the first element with row greater than or equal to in_row.
+    const uword      col_offset = iterator_base::M->col_ptrs[col    ];
+    const uword next_col_offset = iterator_base::M->col_ptrs[col + 1];
 
-    // Is there anything in this new column?
-    for (uword ind = 0; ((iterator_base::M->col_ptrs[cur_col] + ind < iterator_base::M->col_ptrs[cur_col + 1]) && (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] <= cur_row)); ind++)
+    const uword* start_ptr = &iterator_base::M->row_indices[     col_offset];
+    const uword*   end_ptr = &iterator_base::M->row_indices[next_col_offset];
+
+    if (start_ptr != end_ptr)
       {
-      if (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] == cur_row)
+      // Find the first element in the column with row greater than or equal to
+      // the current row.
+      const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row);
+
+      if (pos_ptr != end_ptr)
         {
-        // We have successfully incremented.
-        internal_row = cur_row;
-        iterator_base::internal_col = cur_col;
-        actual_pos = iterator_base::M->col_ptrs[cur_col] + ind;
-
-        return *this; // Now we are done.
+        // We found something in the column, but is the row index correct?
+        if ((*pos_ptr) == internal_row)
+          {
+          // Exact match---so we are done.
+          iterator_base::internal_col = col;
+          actual_pos = col_offset + (pos_ptr - start_ptr);
+          return *this;
+          }
+        else if ((*pos_ptr) < next_min_row)
+          {
+          // The first element in this column is in a subsequent row, but it's
+          // the minimum row we've seen so far.
+          next_min_row = (*pos_ptr);
+          next_min_col = col;
+          next_actual_pos = col_offset + (pos_ptr - start_ptr);
+          }
+        else if ((*pos_ptr) == next_min_row && col < next_min_col)
+          {
+          // The first element in this column is in a subsequent row that we
+          // already have another element for, but the column index is less so
+          // this element will come first.
+          next_min_col = col;
+          next_actual_pos = col_offset + (pos_ptr - start_ptr);
+          }
         }
       }
     }
+
+  // Restart the search in the next row.
+  for (uword col = 0; col <= iterator_base::internal_col; ++col)
+    {
+    // Find the first element with row greater than or equal to in_row + 1.
+    const uword      col_offset = iterator_base::M->col_ptrs[col    ];
+    const uword next_col_offset = iterator_base::M->col_ptrs[col + 1];
+
+    const uword* start_ptr = &iterator_base::M->row_indices[     col_offset];
+    const uword*   end_ptr = &iterator_base::M->row_indices[next_col_offset];
+
+    if (start_ptr != end_ptr)
+      {
+      const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row + 1);
+
+      if (pos_ptr != end_ptr)
+        {
+        // We found something in the column, but is the row index correct?
+        if ((*pos_ptr) == internal_row + 1)
+          {
+          // Exact match---so we are done.
+          iterator_base::internal_col = col;
+          internal_row++;
+          actual_pos = col_offset + (pos_ptr - start_ptr);
+          return *this;
+          }
+        else if ((*pos_ptr) < next_min_row)
+          {
+          // The first element in this column is in a subsequent row, but it's
+          // the minimum row we've seen so far.
+          next_min_row = (*pos_ptr);
+          next_min_col = col;
+          next_actual_pos = col_offset + (pos_ptr - start_ptr);
+          }
+        else if ((*pos_ptr) == next_min_row && col < next_min_col)
+          {
+          // The first element in this column is in a subsequent row that we
+          // already have another element for, but the column index is less so
+          // this element will come first.
+          next_min_col = col;
+          next_actual_pos = col_offset + (pos_ptr - start_ptr);
+          }
+        }
+      }
+    }
+
+  iterator_base::internal_col = next_min_col;
+  internal_row = next_min_row;
+  actual_pos = next_actual_pos;
+
+  return *this; // Now we are done.
   }
 
 
@@ -572,7 +697,7 @@ SpMat<eT>::const_row_iterator::operator++()
  */
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::const_row_iterator
 SpMat<eT>::const_row_iterator::operator++(int)
   {
@@ -594,35 +719,96 @@ arma_hot
 typename SpMat<eT>::const_row_iterator&
 SpMat<eT>::const_row_iterator::operator--()
   {
+  if (iterator_base::internal_pos == 0)
+    {
+    // Do nothing; we are already at the beginning.
+    return *this;
+    }
+
   iterator_base::internal_pos--;
 
-  // We have to search backwards.
-  uword cur_col = iterator_base::internal_col;
-  uword cur_row = internal_row;
+  // We have to search backwards.  We'll do this by going backwards over columns
+  // and seeing if we find an element in the same row.
+  uword max_row = 0;
+  uword max_col = 0;
+  uword next_actual_pos = 0;
 
-  while (true) // This loop is terminated from the inside.
+  //for (uword col = iterator_base::internal_col; col > 1; --col)
+  for (uword col = iterator_base::internal_col; col >= 1; --col)
     {
-    // Decrement the current column and see if we are now on a new row.  This is a uword so a negativity check won't work.
-    if (--cur_col > iterator_base::M->n_cols /* this means it underflew */)
-      {
-      cur_col = iterator_base::M->n_cols - 1;
-      cur_row--;
-      }
+    // Find the first element with row greater than or equal to in_row + 1.
+    const uword      col_offset = iterator_base::M->col_ptrs[col - 1];
+    const uword next_col_offset = iterator_base::M->col_ptrs[col    ];
 
-    // Is there anything in this new column?
-    for (uword ind = 0; ((iterator_base::M->col_ptrs[cur_col] + ind < iterator_base::M->col_ptrs[cur_col + 1]) && (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] <= cur_row)); ind++)
+    const uword* start_ptr = &iterator_base::M->row_indices[     col_offset];
+    const uword*   end_ptr = &iterator_base::M->row_indices[next_col_offset];
+
+    if (start_ptr != end_ptr)
       {
-      if (iterator_base::M->row_indices[iterator_base::M->col_ptrs[cur_col] + ind] == cur_row)
+      // There are elements in this column.
+      const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row + 1);
+
+      if (pos_ptr != start_ptr)
         {
-        // We have successfully decremented.
-        iterator_base::internal_col = cur_col;
-        internal_row = cur_row;
-        actual_pos = iterator_base::M->col_ptrs[cur_col] + ind;
-
-        return *this; // Now we are done.
+        // The element before pos_ptr is the one we are interested in.
+        if (*(pos_ptr - 1) > max_row)
+          {
+          max_row = *(pos_ptr - 1);
+          max_col = col - 1;
+          next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
+          }
+        else if (*(pos_ptr - 1) == max_row && (col - 1) > max_col)
+          {
+          max_col = col - 1;
+          next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
+          }
         }
       }
     }
+
+  // Now loop around to the columns at the end of the matrix.
+  for (uword col = iterator_base::M->n_cols - 1; col >= iterator_base::internal_col; --col)
+    {
+    // Find the first element with row greater than or equal to in_row + 1.
+    const uword      col_offset = iterator_base::M->col_ptrs[col    ];
+    const uword next_col_offset = iterator_base::M->col_ptrs[col + 1];
+
+    const uword* start_ptr = &iterator_base::M->row_indices[     col_offset];
+    const uword*   end_ptr = &iterator_base::M->row_indices[next_col_offset];
+
+    if (start_ptr != end_ptr)
+      {
+      // There are elements in this column.
+      const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row);
+
+      if (pos_ptr != start_ptr)
+        {
+        // There are elements in this column with row index < internal_row.
+        if (*(pos_ptr - 1) > max_row)
+          {
+          max_row = *(pos_ptr - 1);
+          max_col = col;
+          next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
+          }
+        else if (*(pos_ptr - 1) == max_row && col > max_col)
+          {
+          max_col = col;
+          next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
+          }
+        }
+      }
+
+    if (col == 0) // Catch edge case that the loop termination condition won't.
+      {
+      break;
+      }
+    }
+
+  iterator_base::internal_col = max_col;
+  internal_row = max_row;
+  actual_pos = next_actual_pos;
+
+  return *this;
   }
 
 
@@ -632,7 +818,7 @@ SpMat<eT>::const_row_iterator::operator--()
  */
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::const_row_iterator
 SpMat<eT>::const_row_iterator::operator--(int)
   {
@@ -744,10 +930,10 @@ SpValProxy<SpMat<eT> >
 SpMat<eT>::row_iterator::operator*()
   {
   return SpValProxy<SpMat<eT> >(
-    const_row_iterator::internal_row,
-    iterator_base::internal_col,
-    access::rw(*iterator_base::M),
-    &access::rw(iterator_base::M->values[const_row_iterator::actual_pos]));
+      const_row_iterator::internal_row,
+      iterator_base::internal_col,
+      access::rw(*iterator_base::M),
+      &access::rw(iterator_base::M->values[const_row_iterator::actual_pos]));
   }
 
 
@@ -766,7 +952,7 @@ SpMat<eT>::row_iterator::operator++()
 
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::row_iterator
 SpMat<eT>::row_iterator::operator++(int)
   {
@@ -793,7 +979,7 @@ SpMat<eT>::row_iterator::operator--()
 
 template<typename eT>
 inline
-arma_hot
+arma_warn_unused
 typename SpMat<eT>::row_iterator
 SpMat<eT>::row_iterator::operator--(int)
   {
