@@ -52,8 +52,8 @@ template<typename eT> class spdiagview;
 
 template<typename eT> class MapMat;
 template<typename eT> class MapMat_val;
-template<typename eT> class MapMat_elem;
-template<typename eT> class MapMat_svel;
+template<typename eT> class SpMat_MapMat_val;
+template<typename eT> class SpSubview_MapMat_val;
 
 template<typename eT, typename T1>              class subview_elem1;
 template<typename eT, typename T1, typename T2> class subview_elem2;
@@ -101,6 +101,8 @@ class op_find_simple;
 class op_find_unique;
 class op_flipud;
 class op_fliplr;
+class op_reverse_vec;
+class op_reverse_mat;
 class op_real;
 class op_imag;
 class op_nonzeros;
@@ -111,6 +113,7 @@ class op_unique_index;
 class op_diff_default;
 class op_hist;
 class op_chi2rnd;
+class op_roots;
 
 class eop_conj;
 
