@@ -1,11 +1,17 @@
-// Copyright (C) 2011-2016 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup arrayops
@@ -40,14 +46,23 @@ arrayops::copy_small(eT* dest, const eT* src, const uword n_elem)
   switch(n_elem)
     {
     case  9:  dest[ 8] = src[ 8];
+    // fallthrough
     case  8:  dest[ 7] = src[ 7];
+    // fallthrough
     case  7:  dest[ 6] = src[ 6];
+    // fallthrough
     case  6:  dest[ 5] = src[ 5];
+    // fallthrough
     case  5:  dest[ 4] = src[ 4];
+    // fallthrough
     case  4:  dest[ 3] = src[ 3];
+    // fallthrough
     case  3:  dest[ 2] = src[ 2];
+    // fallthrough
     case  2:  dest[ 1] = src[ 1];
+    // fallthrough
     case  1:  dest[ 0] = src[ 0];
+    // fallthrough
     default:  ;
     }
   }
@@ -591,7 +606,7 @@ arrayops::inplace_set(eT* dest, const eT val, const uword n_elem)
   {
   typedef typename get_pod_type<eT>::result pod_type;
   
-  if( (n_elem <= 16) && (is_cx<eT>::no) )
+  if( (n_elem <= 9) && (is_cx<eT>::no) )
     {
     arrayops::inplace_set_small(dest, val, n_elem);
     }
@@ -599,7 +614,7 @@ arrayops::inplace_set(eT* dest, const eT val, const uword n_elem)
     {
     if( (val == eT(0)) && (std::numeric_limits<eT>::is_integer || (std::numeric_limits<pod_type>::is_iec559 && is_real<pod_type>::value)) )
       {
-      std::memset(dest, 0, sizeof(eT)*n_elem);
+      std::memset((void*)dest, 0, sizeof(eT)*n_elem);
       }
     else
       {
@@ -660,22 +675,24 @@ arrayops::inplace_set_small(eT* dest, const eT val, const uword n_elem)
   {
   switch(n_elem)
     {
-    case 16: dest[15] = val;
-    case 15: dest[14] = val;
-    case 14: dest[13] = val;
-    case 13: dest[12] = val;
-    case 12: dest[11] = val;
-    case 11: dest[10] = val;
-    case 10: dest[ 9] = val;
     case  9: dest[ 8] = val;
+    // fallthrough
     case  8: dest[ 7] = val;
+    // fallthrough
     case  7: dest[ 6] = val;
+    // fallthrough
     case  6: dest[ 5] = val;
+    // fallthrough
     case  5: dest[ 4] = val;
+    // fallthrough
     case  4: dest[ 3] = val;
+    // fallthrough
     case  3: dest[ 2] = val;
+    // fallthrough
     case  2: dest[ 1] = val;
+    // fallthrough
     case  1: dest[ 0] = val;
+    // fallthrough
     default:;
     }
   }
