@@ -158,7 +158,7 @@ gmm_diag<eT>::set_params(const Base<eT,T1>& in_means_expr, const Base<eT,T2>& in
   
   const eT s = accu(in_hefts);
   
-  arma_debug_check( ((s < (eT(1) - Datum<eT>::eps)) || (s > (eT(1) + Datum<eT>::eps))), "gmm_diag::set_params(): sum of given hefts is not 1" );
+  arma_debug_check( ((s < (eT(1) - eT(0.001))) || (s > (eT(1) + eT(0.001)))), "gmm_diag::set_params(): sum of given hefts is not 1" );
   
   access::rw(means) = in_means;
   access::rw(dcovs) = in_dcovs;
@@ -230,7 +230,7 @@ gmm_diag<eT>::set_hefts(const Base<eT,T1>& in_hefts_expr)
   
   const eT s = accu(in_hefts);
   
-  arma_debug_check( ((s < (eT(1) - Datum<eT>::eps)) || (s > (eT(1) + Datum<eT>::eps))), "gmm_diag::set_hefts(): sum of given hefts is not 1" );
+  arma_debug_check( ((s < (eT(1) - eT(0.001))) || (s > (eT(1) + eT(0.001)))), "gmm_diag::set_hefts(): sum of given hefts is not 1" );
   
   // make sure all hefts are positive and non-zero
   
@@ -793,7 +793,7 @@ gmm_diag<eT>::learn
     
     reset(X.n_rows, N_gaus);
     
-    if(print_mode)  { get_stream_err2() << "gmm_diag::learn(): generating initial means\n"; get_stream_err2().flush(); }
+    if(print_mode)  { get_cout_stream() << "gmm_diag::learn(): generating initial means\n"; get_cout_stream().flush(); }
     
          if(dist_mode == eucl_dist)  { generate_initial_means<1>(X, seed_mode); }
     else if(dist_mode == maha_dist)  { generate_initial_means<2>(X, seed_mode); }
@@ -804,14 +804,14 @@ gmm_diag<eT>::learn
   
   if(km_iter > 0)
     {
-    const arma_ostream_state stream_state(get_stream_err2());
+    const arma_ostream_state stream_state(get_cout_stream());
     
     bool status = false;
     
          if(dist_mode == eucl_dist)  { status = km_iterate<1>(X, km_iter, print_mode, "gmm_diag::learn(): k-means"); }
     else if(dist_mode == maha_dist)  { status = km_iterate<2>(X, km_iter, print_mode, "gmm_diag::learn(): k-means"); }
     
-    stream_state.restore(get_stream_err2());
+    stream_state.restore(get_cout_stream());
     
     if(status == false)  { arma_debug_warn("gmm_diag::learn(): k-means algorithm failed; not enough data, or too many gaussians requested"); init(orig); return false; }
     }
@@ -823,7 +823,7 @@ gmm_diag<eT>::learn
   
   if(seed_mode != keep_existing)
     {
-    if(print_mode)  { get_stream_err2() << "gmm_diag::learn(): generating initial covariances\n"; get_stream_err2().flush(); }
+    if(print_mode)  { get_cout_stream() << "gmm_diag::learn(): generating initial covariances\n"; get_cout_stream().flush(); }
     
          if(dist_mode == eucl_dist)  { generate_initial_params<1>(X, var_floor_actual); }
     else if(dist_mode == maha_dist)  { generate_initial_params<2>(X, var_floor_actual); }
@@ -834,11 +834,11 @@ gmm_diag<eT>::learn
   
   if(em_iter > 0)
     {
-    const arma_ostream_state stream_state(get_stream_err2());
+    const arma_ostream_state stream_state(get_cout_stream());
     
     const bool status = em_iterate(X, em_iter, var_floor_actual, print_mode);
     
-    stream_state.restore(get_stream_err2());
+    stream_state.restore(get_cout_stream());
     
     if(status == false)  { arma_debug_warn("gmm_diag::learn(): EM algorithm failed"); init(orig); return false; }
     }
@@ -903,7 +903,7 @@ gmm_diag<eT>::kmeans_wrapper
     
     access::rw(means).zeros(X.n_rows, N_gaus);
     
-    if(print_mode)  { get_stream_err2() << "kmeans(): generating initial means\n"; }
+    if(print_mode)  { get_cout_stream() << "kmeans(): generating initial means\n"; }
     
     generate_initial_means<1>(X, seed_mode);
     }
@@ -913,13 +913,13 @@ gmm_diag<eT>::kmeans_wrapper
   
   if(km_iter > 0)
     {
-    const arma_ostream_state stream_state(get_stream_err2());
+    const arma_ostream_state stream_state(get_cout_stream());
     
     bool status = false;
     
     status = km_iterate<1>(X, km_iter, print_mode, "kmeans()");
     
-    stream_state.restore(get_stream_err2());
+    stream_state.restore(get_cout_stream());
     
     if(status == false)  { arma_debug_warn("kmeans(): clustering failed; not enough data, or too many means requested"); return false; }
     }
@@ -1080,7 +1080,7 @@ gmm_diag<eT>::internal_gen_boundaries(const uword N) const
     static const uword n_threads = 1;
   #endif
   
-  // get_stream_err2() << "gmm_diag::internal_gen_boundaries(): n_threads: " << n_threads << '\n';
+  // get_cout_stream() << "gmm_diag::internal_gen_boundaries(): n_threads: " << n_threads << '\n';
   
   umat boundaries(2, n_threads);
   
@@ -1106,7 +1106,7 @@ gmm_diag<eT>::internal_gen_boundaries(const uword N) const
     boundaries.zeros();
     }
   
-  // get_stream_err2() << "gmm_diag::internal_gen_boundaries(): boundaries: " << '\n' << boundaries << '\n';
+  // get_cout_stream() << "gmm_diag::internal_gen_boundaries(): boundaries: " << '\n' << boundaries << '\n';
   
   return boundaries;
   }
@@ -1959,7 +1959,7 @@ gmm_diag<eT>::generate_initial_means(const Mat<eT>& X, const gmm_seed_mode& seed
       }
     }
   
-  // get_stream_err2() << "generate_initial_means():" << '\n';
+  // get_cout_stream() << "generate_initial_means():" << '\n';
   // means.print();
   }
 
@@ -2128,13 +2128,13 @@ gmm_diag<eT>::km_iterate(const Mat<eT>& X, const uword max_iter, const bool verb
   
   if(verbose)
     {
-    get_stream_err2().unsetf(ios::showbase);
-    get_stream_err2().unsetf(ios::uppercase);
-    get_stream_err2().unsetf(ios::showpos);
-    get_stream_err2().unsetf(ios::scientific);
+    get_cout_stream().unsetf(ios::showbase);
+    get_cout_stream().unsetf(ios::uppercase);
+    get_cout_stream().unsetf(ios::showpos);
+    get_cout_stream().unsetf(ios::scientific);
     
-    get_stream_err2().setf(ios::right);
-    get_stream_err2().setf(ios::fixed);
+    get_cout_stream().setf(ios::right);
+    get_cout_stream().setf(ios::fixed);
     }
   
   const uword X_n_cols = X.n_cols;
@@ -2166,7 +2166,7 @@ gmm_diag<eT>::km_iterate(const Mat<eT>& X, const uword max_iter, const bool verb
     const uword n_threads = 1;
   #endif
   
-  if(verbose)  { get_stream_err2() << signature << ": n_threads: " << n_threads  << '\n'; get_stream_err2().flush(); }
+  if(verbose)  { get_cout_stream() << signature << ": n_threads: " << n_threads  << '\n'; get_cout_stream().flush(); }
   
   for(uword iter=1; iter <= max_iter; ++iter)
     {
@@ -2282,7 +2282,7 @@ gmm_diag<eT>::km_iterate(const Mat<eT>& X, const uword max_iter, const bool verb
     
     if(dead_gs.n_elem > 0)
       {
-      if(verbose)  { get_stream_err2() << signature << ": recovering from dead means\n"; get_stream_err2().flush(); }
+      if(verbose)  { get_cout_stream() << signature << ": recovering from dead means\n"; get_cout_stream().flush(); }
       
       uword* last_indx_mem = last_indx.memptr();
     
@@ -2328,16 +2328,16 @@ gmm_diag<eT>::km_iterate(const Mat<eT>& X, const uword max_iter, const bool verb
     
     if(verbose)
       {
-      get_stream_err2() << signature << ": iteration: ";
-      get_stream_err2().unsetf(ios::scientific);
-      get_stream_err2().setf(ios::fixed);
-      get_stream_err2().width(std::streamsize(4));
-      get_stream_err2() << iter;
-      get_stream_err2() << "   delta: ";
-      get_stream_err2().unsetf(ios::fixed);
-      //get_stream_err2().setf(ios::scientific);
-      get_stream_err2() << rs_delta.mean() << '\n';
-      get_stream_err2().flush();
+      get_cout_stream() << signature << ": iteration: ";
+      get_cout_stream().unsetf(ios::scientific);
+      get_cout_stream().setf(ios::fixed);
+      get_cout_stream().width(std::streamsize(4));
+      get_cout_stream() << iter;
+      get_cout_stream() << "   delta: ";
+      get_cout_stream().unsetf(ios::fixed);
+      //get_cout_stream().setf(ios::scientific);
+      get_cout_stream() << rs_delta.mean() << '\n';
+      get_cout_stream().flush();
       }
     
     arma::swap(old_means, new_means);
@@ -2369,13 +2369,13 @@ gmm_diag<eT>::em_iterate(const Mat<eT>& X, const uword max_iter, const eT var_fl
   
   if(verbose)
     {
-    get_stream_err2().unsetf(ios::showbase);
-    get_stream_err2().unsetf(ios::uppercase);
-    get_stream_err2().unsetf(ios::showpos);
-    get_stream_err2().unsetf(ios::scientific);
+    get_cout_stream().unsetf(ios::showbase);
+    get_cout_stream().unsetf(ios::uppercase);
+    get_cout_stream().unsetf(ios::showpos);
+    get_cout_stream().unsetf(ios::scientific);
     
-    get_stream_err2().setf(ios::right);
-    get_stream_err2().setf(ios::fixed);
+    get_cout_stream().setf(ios::right);
+    get_cout_stream().setf(ios::fixed);
     }
   
   const umat boundaries = internal_gen_boundaries(X.n_cols);
@@ -2402,7 +2402,7 @@ gmm_diag<eT>::em_iterate(const Mat<eT>& X, const uword max_iter, const eT var_fl
   
   if(verbose)
     {
-    get_stream_err2() << "gmm_diag::learn(): EM: n_threads: " << n_threads  << '\n';
+    get_cout_stream() << "gmm_diag::learn(): EM: n_threads: " << n_threads  << '\n';
     }
   
   eT old_avg_log_p = -Datum<eT>::inf;
@@ -2419,16 +2419,16 @@ gmm_diag<eT>::em_iterate(const Mat<eT>& X, const uword max_iter, const eT var_fl
     
     if(verbose)
       {
-      get_stream_err2() << "gmm_diag::learn(): EM: iteration: ";
-      get_stream_err2().unsetf(ios::scientific);
-      get_stream_err2().setf(ios::fixed);
-      get_stream_err2().width(std::streamsize(4));
-      get_stream_err2() << iter;
-      get_stream_err2() << "   avg_log_p: ";
-      get_stream_err2().unsetf(ios::fixed);
-      //get_stream_err2().setf(ios::scientific);
-      get_stream_err2() << new_avg_log_p << '\n';
-      get_stream_err2().flush();
+      get_cout_stream() << "gmm_diag::learn(): EM: iteration: ";
+      get_cout_stream().unsetf(ios::scientific);
+      get_cout_stream().setf(ios::fixed);
+      get_cout_stream().width(std::streamsize(4));
+      get_cout_stream() << iter;
+      get_cout_stream() << "   avg_log_p: ";
+      get_cout_stream().unsetf(ios::fixed);
+      //get_cout_stream().setf(ios::scientific);
+      get_cout_stream() << new_avg_log_p << '\n';
+      get_cout_stream().flush();
       }
     
     if(arma_isfinite(new_avg_log_p) == false)  { return false; }
